// Models.h
// ,   , , 
//   ,    
// .  .

#ifndef __Models_h
#define __Models_h

#include "Auto-Lib.h"
#include "xTypes.h"
#include "xString.h"
#include <vector>

//     
int	get_uint32(uint8** parr);
char get_char(uint8** parr);
flt	get_flt(uint8** parr);
char *get_str(uint8 **parr);

//    5-  ,
//3  -   , 2  -  
//
struct TmVertex
{
	flt x,y,z,u,v;
};

//    3-  ,
//3  -   
struct TxVector3f
{
	flt x,y,z;
};


//,   
struct TmFrame
{
	TmVertex *Vertices;
};

//,   
struct TmFace
{
	uint32 a,b,c;
};

//,   
struct TmGroup
{
	uint32 GroupID;
	uint32 GroupIndex;
	uint32 PreFrameCount;
	uint32 LoopFrameCount;
	uint32 PostFrameCount;
	uint32 PreTime;
	uint32 LoopTime;
	uint32 PostTime;
	uint32 PreAndLoopTime;
	uint32 AllTime;
	TmFrame	**Frames;
	uint32 *Time;
};

//,   
struct TmPart
{
	uint32 VertexCount;
	uint32 FacesCount;
	TmFace *Faces;
	uint32 FramesCount;
	TmFrame *Frames;
	uint32 GroupsCount;
	TmGroup *Groups;
	TmGroup IdleGroup;
	
	TmFrame CurrentFrame;
	TmGroup *CurrentGroup;//  ""
	TmGroup *NextGroup;// ,    ,
						//   
	uint32 LocalTime; //    
	TmFrame *PrevFrame; // 
	uint32 NextFrameIndex;
	TmFrame TmpFrame;// ,   
				//    
					//  
	uint32 AfterPrevTime;
};

//,   
const uint32 
  fire1=1, fire2=2, moveleft=4, moveright=8, 
	moveup=16, movedown=32, movefwd=64, moveback=128,
	rotleft=256, rotright=512, rotup=1024, rotdown=2048,
	rotcw=4096, rotccw=8192, pain=16384,
	fire=32768, move=65536,rot=131072;

// 
const xstring Caption[]=
{
  "1","2","","",
   "","","","",
   ".",".",".",".",
   " "," ","",
   "", "",""
};

class TmLoaderAutomat;
class TmPartAutomat;

// ''
class TmModelAutomat : public LoggedAutomat
{

public:

	xstring ModelName;
	TmGroup* GetNextGroup(TmPart *Part);
	xstring Name;
	uint32 PartCount;
	TmPart *Parts;
	uint32 BoundVertexCount;
	uint32 BoundFacesCount;
	TmFace *BoundFaces;
	TxVector3f *BoundVertices;
	uint32 LastTime;
	uint32 DeltaTime;
	uint32 Version;
	uint32 TextureIndex;
	
	TmLoaderAutomat *A1;
	TmPartAutomat **A2;
	
	TmModelAutomat(xstring ModelName,uint32 a, xstring szAutomatDescription,BaseLogger *Logger);
	virtual void Main();
	virtual void Action(uint32 n);
	virtual bool Param(uint32 n);
	
	void createloader();
	void destroyloader();
	void createparts();
	void destroyparts();
	void load();
	void unload();
};

// ''
class TmLoaderAutomat : public LoggedAutomat
{

public:

	TmLoaderAutomat(TmModelAutomat *Model,uint32 a,  xstring szAutomatDescription,BaseLogger *Logger);
	virtual void Main();
	bool Param(uint32 n);
	void Action(uint32 n);
	void load();
	void unload();

	TmModelAutomat *Model;
	bool x7;
};

// ''
class TmPartAutomat : public LoggedAutomat
{

public:
 
	TmPartAutomat(TmPart *Part,TmModelAutomat *Model,uint32 a, xstring szAutomatDescription, BaseLogger *Logger);
	virtual void Main();
	virtual bool Param(uint32 n);
	virtual void Action(uint32 n);
	void CalculateFrame(bool DefState);
	void IncNFI(bool DefState);
	void InterpolateFrame(TmFrame *dest,TmFrame *s1,TmFrame *s2,uint32 alltime,uint32 delta, uint32 pn);
	TmGroup* GetNextGroup();
	TmPart *Part;
	TmModelAutomat *Model;
	uint32 DeltaTime;
};

// '   A0 '''
class TmModelEvent : public DescriptedEvent
{
public:
  TmModelEvent(uint32 flag, uint32 time, bool mode, uint32 x, xstring szEventDescription);
  bool mode;
  uint32 flag;
  uint32 time;
};
#endif