// ModelessDlg.c
#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "resource.h"

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK AboutDiaProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK PenWidthDiaProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK PenColorDiaProc(HWND, UINT, WPARAM, LPARAM);

char szWinName[] = "MyWin";
char szWinTitle[] = "Modeless Dialog";

HWND hPColorDlg;
int penWidth;
int redSliderPos;
int greenSliderPos;
int blueSliderPos;

///////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPreInst,
				   LPSTR lpCmdLine, int nCmdShow)	
{
	HWND hWnd;  
	MSG lpMsg;
	WNDCLASS wcApp;
	
	if(!hPreInst)
	{
		wcApp.lpszClassName = szWinName;
		wcApp.hInstance = hInst;
		wcApp.lpfnWndProc = WndProc;
		wcApp.hCursor = LoadCursor(NULL, IDC_ARROW);
		wcApp.hIcon = NULL;
		wcApp.lpszMenuName = MAKEINTRESOURCE(IDR_MENU1);
		wcApp.hbrBackground = GetStockObject(WHITE_BRUSH);
		wcApp.style = CS_HREDRAW | CS_VREDRAW;
		wcApp.cbClsExtra = 0;
		wcApp.cbWndExtra = 0;

		if (!RegisterClass(&wcApp))
			return 0;
	}

	hWnd = CreateWindow(szWinName, szWinTitle,
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT, CW_USEDEFAULT,
				CW_USEDEFAULT, CW_USEDEFAULT,
				(HWND) NULL, (HMENU) NULL,
				(HINSTANCE)hInst, (LPVOID)NULL);
	if (!hWnd) 
		return FALSE;  

	InitCommonControls();
	
	ShowWindow(hWnd, nCmdShow); 
	UpdateWindow(hWnd);

	while (GetMessage(&lpMsg, NULL, 0, 0))  
	{
		TranslateMessage(&lpMsg);
		DispatchMessage(&lpMsg);
	}

	return (lpMsg.wParam);
}
///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK AboutDiaProc(HWND hDlg, UINT messg, 
			 WPARAM wParam, LPARAM lParam)
{
	switch (messg)
	{
	case WM_INITDIALOG:
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			EndDialog(hDlg, 0);
			break;
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK PenWidthDiaProc(HWND hDlg, UINT messg, 
			 WPARAM wParam, LPARAM lParam)
{
	switch (messg)
	{
	case WM_INITDIALOG:
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:			
			InvalidateRect(hDlg, NULL, TRUE);
			EndDialog(hDlg, TRUE);	
			break;

		case IDC_WIDTH1:
			penWidth = 1;
			CheckRadioButton(hDlg, IDC_WIDTH1, IDC_WIDTH5, wParam);
			break;
		case IDC_WIDTH2:
			penWidth = 2;
			CheckRadioButton(hDlg, IDC_WIDTH1, IDC_WIDTH5, wParam);
			break;
		case IDC_WIDTH5:
			penWidth = 5;
			CheckRadioButton(hDlg, IDC_WIDTH1, IDC_WIDTH5, wParam);
			break;
		case IDC_WIDTH10:
			penWidth = 10;
			CheckRadioButton(hDlg, IDC_WIDTH1, IDC_WIDTH5, wParam);
			break;
		case IDC_WIDTH20:
			penWidth = 20;
			CheckRadioButton(hDlg, IDC_WIDTH1, IDC_WIDTH5, wParam);
			break;
		default:
			return FALSE;
		}
		break;

	default:
		return FALSE;
	}

	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK PenColorDiaProc(HWND hDlg, UINT messg, 
			 WPARAM wParam, LPARAM lParam)
{
	HINSTANCE hInst;
	HWND parentWnd;
	static HWND hRedTrackbarWnd;
	static HWND hGreenTrackbarWnd;
	static HWND hBlueTrackbarWnd;
	
	switch (messg)
	{
    case WM_INITDIALOG: 
		hInst = (HINSTANCE)GetModuleHandle(NULL);
		//TrackBar for 'Red'
		hRedTrackbarWnd = CreateWindow(TRACKBAR_CLASS, szWinName, 
			WS_CHILD|WS_VISIBLE|WS_TABSTOP|TBS_AUTOTICKS,
			50, 10, 150, 30, hDlg, NULL, hInst, NULL);
		SendMessage(hRedTrackbarWnd, TBM_SETRANGE, TRUE, MAKELONG(0, 255));
		SendMessage(hRedTrackbarWnd, TBM_SETTICFREQ, 32, 0);
		SendMessage(hRedTrackbarWnd, TBM_SETPOS, TRUE, 127);
		SetDlgItemInt(hDlg, IDC_EDIT_RED, 127, TRUE);
		//TrackBar for 'Green'
		hGreenTrackbarWnd = CreateWindow(TRACKBAR_CLASS, szWinName, 
			WS_CHILD|WS_VISIBLE|WS_TABSTOP|TBS_AUTOTICKS,
			50, 60, 150, 30, hDlg, NULL, hInst, NULL);
		SendMessage(hGreenTrackbarWnd, TBM_SETRANGE, TRUE, MAKELONG(0, 255));
		SendMessage(hGreenTrackbarWnd, TBM_SETTICFREQ, 32, 0);
		SendMessage(hGreenTrackbarWnd, TBM_SETPOS, TRUE, 127);
		SetDlgItemInt(hDlg, IDC_EDIT_GREEN, 127, TRUE);
		//TrackBar for 'Blue'
		hBlueTrackbarWnd = CreateWindow(TRACKBAR_CLASS, szWinName, 
			WS_CHILD|WS_VISIBLE|WS_TABSTOP|TBS_AUTOTICKS,
			50, 110, 150, 30, hDlg, NULL, hInst, NULL);
		SendMessage(hBlueTrackbarWnd, TBM_SETRANGE, TRUE, MAKELONG(0, 255));
		SendMessage(hBlueTrackbarWnd, TBM_SETTICFREQ, 32, 0);
		SendMessage(hBlueTrackbarWnd, TBM_SETPOS, TRUE, 127);
		SetDlgItemInt(hDlg, IDC_EDIT_BLUE, 127, TRUE);

		return TRUE; 

	case WM_HSCROLL:
		redSliderPos = SendMessage(hRedTrackbarWnd, TBM_GETPOS, 0, 0);
		SetDlgItemInt(hDlg, IDC_EDIT_RED, redSliderPos, TRUE);
		greenSliderPos = SendMessage(hGreenTrackbarWnd, TBM_GETPOS, 0, 0);
		SetDlgItemInt(hDlg, IDC_EDIT_GREEN, greenSliderPos, TRUE);
		blueSliderPos = SendMessage(hBlueTrackbarWnd, TBM_GETPOS, 0, 0);
		SetDlgItemInt(hDlg, IDC_EDIT_BLUE, blueSliderPos, TRUE);

		parentWnd = GetParent(hDlg);
		InvalidateRect(parentWnd, NULL, TRUE);
		
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		default:
			return FALSE;
		}
		break;
		
	default:
		return FALSE;
	}

	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc(HWND hWnd, UINT messg,
			 WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	static HPEN hPen;
	static COLORREF penColor;

	RECT wnd0_rect;		//v ,    
	static int x0, y0;	//v     
	static int w0, h0;	//  v  
	RECT PColorDlg_rect;//v ,   PColorDlg 
	int xDlg, yDlg;		//v     PColorDlg
	int wDlg, hDlg;		//  v  PColorDlg
	const int r = 100;  // 

	switch (messg)
	{
	case WM_CREATE:
		hPColorDlg = CreateDialog((HINSTANCE)GetModuleHandle(NULL),
			   MAKEINTRESOURCE(IDD_PEN_COLOR), hWnd, (DLGPROC)PenColorDiaProc);
		break;

	case WM_SIZE: case WM_MOVE:

		GetWindowRect(hWnd, &wnd0_rect);
		x0 = wnd0_rect.left;	
		y0 = wnd0_rect.top;
		w0 = wnd0_rect.right - x0;
		h0 = wnd0_rect.bottom - y0;

		GetWindowRect(hPColorDlg, &PColorDlg_rect);
		xDlg = PColorDlg_rect.left;	
		yDlg = PColorDlg_rect.top;
		wDlg = PColorDlg_rect.right - xDlg;
		hDlg = PColorDlg_rect.bottom - yDlg;

		MoveWindow(hPColorDlg, x0 + w0 - wDlg-5, y0 + 45, wDlg, hDlg, TRUE);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDM_ABOUT:
			DialogBox((HINSTANCE)GetModuleHandle(NULL),
				MAKEINTRESOURCE(IDD_ABOUT), hWnd, AboutDiaProc);
			break;
		case IDM_PEN_WIDTH:
			DialogBox((HINSTANCE)GetModuleHandle(NULL),
				MAKEINTRESOURCE(IDD_PEN_WIDTH), hWnd, PenWidthDiaProc);			
			break;
		case IDM_EXIT:
			SendMessage(hWnd, WM_CLOSE, 0, 0L);
			break;
		default:
			break;
		}

		InvalidateRect(hWnd, NULL, TRUE);
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);

		penColor = RGB(redSliderPos, greenSliderPos, blueSliderPos);

		hPen = CreatePen(0, penWidth, penColor);
		SelectObject(hdc, hPen);

		Ellipse(hdc, w0/2 - r, h0 - 70 - 2*r, w0/2 + r , h0 - 70);

		EndPaint(hWnd, &ps);
		break;

	case WM_DESTROY:
		DeleteObject(hPen);
		DestroyWindow(hPColorDlg);
		PostQuitMessage(0);
		break;
	
	default:
		return (DefWindowProc(hWnd, messg, wParam, lParam));
	}

	return (0);
}